Exception: Money::Currency::MissingAttributeError

Inherits:
StandardError
  • Object
show all
Defined in:
lib/money/currency.rb,
sig/lib/money/currency.rbs

Overview

Thrown when a Currency has been registered without all the attributes which are required for the current action.

Instance Method Summary collapse

Constructor Details

#initialize(method, currency, attribute) ⇒ MissingAttributeError

Returns a new instance of MissingAttributeError.

Parameters:

  • method (Object)
  • currency (Object)
  • attribute (Object)


24
25
26
27
28
29
# File 'lib/money/currency.rb', line 24

def initialize(method, currency, attribute)
  super(
    "Can't call Currency.#{method} - currency '#{currency}' is missing " \
    "the attribute '#{attribute}'"
  )
end